#!/bin/bash
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Check DB is accessible and schema is loaded OK

# Source function library.
. /etc/rc.d/init.d/functions

echo -n "Checking for DB connection details...  "
FILE=`ls -1 /opt/glite/etc/glite-transfer-agent-fts-*.properties.xml 2>/dev/null`
if [ $? -eq 0 ]; then
    DBCONNECTSTRING=$(xsltproc extractdbconnection.xsl ${FILE} 2>/dev/null)
    if [ $? -ne 0  ]; then
	echo_failure
	echo
	echo Failed extracting DB connect details from: ${FILE}
	exit 1
    fi
    echo_success
    echo
else
    echo_failure
    echo
    echo "Cannot find any config files of form: /opt/glite/etc/glite-transfer-agent-fts-*.properties.xml"
    exit 1
fi
    
echo -n "Checking connection to DB is alive... "
sqlplus -S -L ${DBCONNECTSTRING} <<EOF >/dev/null 2>&1
exit
EOF
if [ $? -eq 0 ]; then
    echo_success
    echo	
else
    echo_failure
    echo
    echo -e "Cannot connect using:\n\tsqlplus -S -L ${DBCONNECTSTRING}"
    exit 1
fi

echo -n "Checking if schema is valid... "
resp=`sqlplus -S -L ${DBCONNECTSTRING} <<EOF 2>/dev/null
select table_name from user_tables;
exit;
EOF`
schemaok="OK"
echo $resp | grep " T_FILE " >/dev/null 2>&1
if [ $? -ne 0 ]; then
    schemaok="T_FILE"
fi
echo $resp | grep " T_FILE_TRANSFER " >/dev/null 2>&1
if [ $? -ne 0 ]; then
    schemaok="T_FILE_TRANSFER"
fi
echo $resp | grep " T_LOGICAL " >/dev/null 2>&1
if [ $? -ne 0 ]; then
    schemaok="T_LOGICAL"
fi
echo $resp | grep " T_CHANNEL " >/dev/null 2>&1
if [ $? -ne 0 ]; then
    schemaok="T_CHANNEL"
fi
echo $resp | grep " T_SCHEMA_VERS " >/dev/null 2>&1
if [ $? -ne 0 ]; then
    schemaok="T_SCHEMA_VERS"
fi
echo $resp | grep " T_JOB " >/dev/null 2>&1
if [ $? -ne 0 ]; then
    schemaok="T_JOB"
fi

if [ $schemaok == "OK" ]; then
    echo_success
    echo 	
else
    echo_failure
    echo
    echo Could not find table: ${schemaok}
    exit 1
fi

echo -n "Checking if schema version is valid... "
resp=`sqlplus -S -L ${DBCONNECTSTRING} <<EOF 2>/dev/null
select major||minor||patch from t_schema_vers;
exit;
EOF`
schemaok="OK"
echo $resp | grep "1310" >/dev/null 2>&1
if [ $? -ne 0 ]; then
    echo_failure
    echo
    echo "Schema version is not 1.3.10"
    exit 1
else
    echo_success
    echo 	
fi
