#!/bin/bash
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Check FTS config files are OK - i.e. well formed with all the necessary bits specified

# Source function library.
. /etc/rc.d/init.d/functions

if [ $# -eq 1 ]; then
    FILE=$1
else
    echo -n "Checking that FTS agent config file exists..."
    FILE=`ls -1 /opt/glite/etc/glite-transfer-agent-fts-*.properties.xml >/dev/null 2>&1`
    if [ $? -eq 0 ]; then
	echo_success
	echo	
    else
	echo_failure
	echo
	exit 1
    fi
    echo -n "Checking that FTS agent config file is unique..."
    numfile=`ls -1 /opt/glite/etc/glite-transfer-agent-fts-*.properties.xml | wc -l 2>/dev/null`
    if [ ${numfile} -eq 1 ]; then
	echo_success
	echo
    else
	echo_failure
	echo
	echo There are ${numfile} config files:
	ls -1 /opt/glite/etc/glite-transfer-agent-fts-*.properties.xml
	exit 1
    fi

fi
    
echo -n "Checking that FTS agent config file is well formed... "

xmllint --relaxng config.rng --noout ${FILE} > /dev/null 2>&1

result=$?

if [ ${result} -eq 0 ]; then
    echo_success
    echo
else
    echo_failure
    echo
    echo xmllint relaxng checking of file: ${FILE}
    xmllint --relaxng config.rng ${FILE} >> ./fts-agent-relaxng-bad.txt 2>&1
    echo Dumping report to: ./fts-agent-relaxng-bad.txt
fi
