#!/usr/bin/python
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# To get various summary information for the given time minus 24 hours.

import sys
import summary
import time

if len(sys.argv) !=4:
    print """Usage: get-summary.py endtime compdir faildir
Required paramaters:
\tendtime\t\tThe end time range, format 2005-08-04 01:32:02
\tcompdir\t\tThe completed directory to scan
\tfaildir\t\tThe failed directory to scan"""
    sys.exit(1)

end = sys.argv[1]
compdir = sys.argv[2]
faildir = sys.argv[3]

scan = summary.FilesInRange()
scan.setRangeOffset(end, 24)
okentries = scan.entriesInRange(compdir)
badentries = scan.entriesInRange(faildir)

print "Scanned %i completed entries from directory %s in time range %s to %s" % (len(okentries), compdir, time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(scan.start)), time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(scan.end)))

print "Scanned %i failed entries from directory %s in time range %s to %s" % (len(badentries), faildir, time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(scan.start)), time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(scan.end)))

# extract ones that habitually failed
uniqueSet = { }
for bad in badentries:
    key=bad.srmsource+bad.srmdest
    if not uniqueSet.has_key(key):
        uniqueSet[key] = bad
for good in okentries:
    key=good.srmsource+good.srmdest
    if uniqueSet.has_key(key):
        del uniqueSet[key]

reallyBadEntries = uniqueSet.values()

#get mean number of attempts
attemptedSet = { }
biglist = badentries + okentries
for bad in biglist:
    key=bad.srmsource+bad.srmdest
    if not attemptedSet.has_key(key):
        attemptedSet[key] = (1, bad)
    else:
        tup = attemptedSet[key]
        notimes = tup[0]
        oldbad = tup[1]
        notimes = notimes + 1
        attemptedSet[key] = (notimes, oldbad)
attempts=attemptedSet.values()

# Dump the completed / fail time
print "Writing %i completed times to time-comp.dat..." % len(okentries)
c1 = open('time-comp.dat', 'w')
if len(okentries) == 0:
    c1.write("-99999999.0\n")
for transfer in okentries:
    c1.write(`summary.hourconvert(scan.start, transfer.epochtime)` + "\n")
c1.close()

print "Writing %i failed times to time-fail.dat..." % len(badentries)
c2 = open('time-fail.dat', 'w')
if len(badentries) == 0:
    c2.write("-99999999.0\n")
for transfer in badentries:
    c2.write(`summary.hourconvert(scan.start, transfer.epochtime)` + "\n")
c2.close()

# Dump the completed / fail time - bad failings
print "Writing %i terminal failed times to time-fail-terminal.dat..." % len(reallyBadEntries)
c2 = open('time-fail-terminal.dat', 'w')
if len(reallyBadEntries) == 0:
    c2.write("-99999999.0\n")
for transfer in reallyBadEntries:
    c2.write(`summary.hourconvert(scan.start, transfer.epochtime)` + "\n")
c2.close()


# Dump the attempts per file
print "Writing %i #attempt times to attempts.dat..." % len(attempts)
c2 = open('attempts.dat', 'w')
if len(attempts) == 0:
    c2.write("-99999999.0\n")
for transfer in attempts:
    c2.write(`summary.hourconvert(scan.start, transfer[1].epochtime)` + ", " + `transfer[0]` + "\n")
c2.close()


# Dump failure classes as function of time
print "Writing failure classes to class-fail.dat..."
c3 = open('class-fail.dat', 'w')
if len(badentries) == 0:
    c3.write("-99999999.0, 0\n")
for transfer in badentries:
    line = `summary.hourconvert(scan.start, transfer.epochtime)` + ", " + `transfer.classcode` + "\n"
    c3.write(line)
c3.close()

# Dump summary failure class as fn of time
print "Writing summary failure classes to class-summary-fail.dat..."
c4 = open('class-summary-fail.dat', 'w')
if len(badentries) == 0:
    c4.write("-99999999.0, 0\n")
for transfer in badentries:
    if transfer.finalclass == 'SUCCESS':
        sumcode = 0
    elif transfer.finalclass == 'SRM_DEST':
        sumcode = 2
    elif transfer.finalclass == 'SRM_SOURCE':
        sumcode = 1
    elif transfer.finalclass == 'TRANSFER':
        sumcode = 3
    elif transfer.finalclass == 'ABORTED' or transfer.finalclass == 'ABORT':
        if transfer.subclass == 'SRMGET' or transfer.subclass == 'SRMGETDONE':
            sumcode = 1
        elif transfer.subclass == 'SRMPUT' or transfer.subclass == 'SRMPUTDONE':
            sumcode = 2
        else:
            sumcode = 3
    elif transfer.finalclass == 'SRM_DONE_DEST':
        sumcode = 2
    elif transfer.finalclass == 'SRM_DONE_SOURCE':
        sumcode = 1
    elif transfer.finalclass == 'TRANSPORT':
        sumcode = 3
    elif transfer.finalclass == 'NETWORK':
        sumcode = 3
    elif transfer.finalclass == 'SYNTAX':
        sumcode = 3
    elif transfer.finalclass == 'SECURITY':
        sumcode = 3
    else:
        sumcode = 3
    line = `summary.hourconvert(scan.start, transfer.epochtime)` + ", " + `sumcode` + "\n"
    c4.write(line)
c4.close()
        
