# Python module to get time diffs
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import summary
import time
import calendar

def toMillis(delta):
    micro = delta.microseconds + (1000000 * delta.seconds) + (3600*24*1000000 * delta.days)
    return micro / 1000.0
    return micro
 
if len(sys.argv) < 3 or len(sys.argv) > 4:
    print """Usage: get-times.py endtime dir [hours]
Required paramaters:
\tendtime\t\tThe end time range, format 2005-08-04 01:32:02
\tcompdir\t\tThe directory to scan
\tcompdir\t\tHours behind the end time to scan (default 24)
"""
    sys.exit(1)

end = sys.argv[1]
dir = sys.argv[2]
if len(sys.argv) == 4:
    hours=sys.argv[3]
else:
    hours=24

scan = summary.FilesInRange()
scan.setRangeOffset(end, hours)
print "Scanning entries from directory %s in time range %s to %s" % ( dir, scan.startDate.strftime("%Y-%m-%d %H:%M:%S"), scan.endDate.strftime("%Y-%m-%d %H:%M:%S") )
entries = scan.entriesInRange(dir, True)
 
print "Scanned %i entries from directory %s in time range %s to %s" % (len(entries), dir, scan.startDate.strftime("%Y-%m-%d %H:%M:%S"), scan.endDate.strftime("%Y-%m-%d %H:%M:%S")) 

print "Writing timings to timings.dat..."
file1 = open('timings.dat', 'w')
for entry in entries:
    srmput=0
    srmget=0
    transfer=0
    srmgetdone=0
    srmputdone=0
    total=0
    source='not-found'
    dest='not-found'
    startTime='unknown-start-time'
    try:
        srmget=toMillis(entry.srmGetEnd-entry.srmGetStart)
    except AttributeError:
        pass
    try:
        srmput=toMillis(entry.srmPutEnd-entry.srmPutStart)
    except AttributeError:
        pass
    try:
        transfer=toMillis(entry.transferEnd-entry.transferStart)
    except AttributeError:
        pass
    try:
        srmgetdone=toMillis(entry.srmGetDoneEnd-entry.srmGetDoneStart)
    except AttributeError:
        pass
    try:
        srmputdone=toMillis(entry.srmPutDoneEnd-entry.srmPutDoneStart)
    except AttributeError:
        pass
    try:
        total=toMillis(entry.doneTime-entry.startTime)
    except AttributeError:
        pass
    try:
        source=entry.turlSouceHost()
    except AttributeError:
        pass
    try:
        dest=entry.turlDestHost()
    except AttributeError:
        pass
    try:
        startTime=calendar.timegm((entry.startTime.timetuple()))
    except AttributeError:
        pass

    file1.write(entry.shortFilename + ", " + source + ", " + dest + ", " + `startTime` +
                ", " + `srmget` + ", " + `srmput` + ", " +
                `transfer` + ", " + `srmgetdone` + ", " + `srmputdone` + ", " + `total` + "\n")
    
if len(entries) == 0:
    file1.write("-99999999.0\n")
file1.close()
