#!/bin/bash
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

LOGDIR=/var/tmp/glite-url-copy-sc3

# Script to make the summary plots

if [ $# != 2 ]; then
    echo -e "Usage:\t$0 endTime channelName\n\n\tendTime\tA UTC dateTime in the form 2005-07-03 08:00:00\n\tchannelName\tThe channel name to process"
    exit 1
fi

DAT=$1
CHANNEL=$2

echo "Processing data for channel $2 back 24 hours from dateTime $1 UTC..."

CHANNELFAIL=${LOGDIR}/${CHANNEL}failed
CHANNELOK=${LOGDIR}/${CHANNEL}completed

if [ -d ${CHANNELOK} ]; then
    echo "Processing completed directory: ${CHANNELOK}"
else
    echo "Cannot find completed directory: ${CHANNELOK}!"
    exit 2
fi

if [ -d ${CHANNELFAIL} ]; then
    echo "Processing failed directory: ${CHANNELFAIL}"
else
    echo "Cannot find failed directory: ${CHANNELFAIL}!"
    exit 3
fi

# Run the data collection script
rm -f time-fail.dat
rm -f time-comp.dat
rm -f class-fail.dat
./get-summary.py "${DAT}"  ${CHANNELOK} ${CHANNELFAIL}

# Run the plotting script
rm -f *.eps
rm -f *.jpg

DATESTRING=`date -d "${DAT}" +"%A %d %B %H.%M.%S"`
export thedate="Channel $CHANNEL for 24 hours ending ${DATESTRING} UTC"
/usr/libexec/CERNLIB/2004/bin/paw -b get-summary.kumac
unset thedate
rm -f last.kumacold

# convert to jpg
for i in *.eps ; do convert -resize 80%x80% $i `echo $i | sed 's/\.eps/\.png/g'` ; done
#for i in *.eps ; do convert -resize 30%x30% $i `echo $i | sed 's/\.eps/\-thumbnail.png/g'` ; done

#some stats
TOTFAIL=`wc -l time-fail.dat | awk '{print $1}'`
TOTCOMP=`wc -l time-comp.dat | awk '{print $1}'` 

# make the dir.
SHORTDATE=`date -d "${DAT}" +%Y%m%d%H%M%S`
if [ ! -d dailyplots-${SHORTDATE} ]; then
    mkdir dailyplots-${SHORTDATE}
fi

# make the webpage
for i in *.eps ; do mv $i dailyplots-${SHORTDATE}/${CHANNEL}-$i ; done
for i in *.png ; do mv $i dailyplots-${SHORTDATE}/${CHANNEL}-$i ; done

cat > dailyplots-${SHORTDATE}/${CHANNEL}.html <<EOF
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
  <head>
    <title>Daily plots for channel ${CHANNEL} ending ${DATESTRING} UTC</title>
  </head>

  <body>
    <h2>Daily plots for channel ${CHANNEL} ending ${DATESTRING} UTC</h2>

<h3>Summary</h3>
<p>
Total completed transfers: ${TOTCOMP}<br>
Total failed transfers:  ${TOTFAIL}
</p>

<h3>Summary plots</h3>
<p>
Note that in these plots hour=24 corresponds to ${DATESTRING} UTC; hour=0
corresponds to 24 hours before that.
</p>
<p>
<table border="0">
<caption>% of individual transfers completed and failed (time)</caption>
<tr><td><img src="${CHANNEL}-perccompleted.png" width="100%"></td><td><img src="${CHANNEL}-percfailed.png" width="100%"></td></tr>
</table>
<table border="0">
<caption>Number of individual transfers completed and failed (time)</caption>
<tr><td><img src="${CHANNEL}-completed.png" width="100%"></td><td><img src="${CHANNEL}-failed.png" width="100%"></td></tr>
</table>
<table border="0">
<caption>Summary failure class (time)</caption>
<tr><td><img src="${CHANNEL}-failure-class-summary.png" width="100%"></td></tr>
</table>
<table border="0">
<caption>Mean attempts per source/dest file pair (time of 1st attempt)</caption>
<tr><td><img src="${CHANNEL}-meanattempts.png" width="100%"></td><td><img src="${CHANNEL}-meanattemptsexclu.png" width="100%"></td></tr>
</table>

</p>

<h3>Detailed plots</h3>
<p>
<table border="0">
<caption>Individual failure class (time)</caption>
<tr><td><img src="${CHANNEL}-failure-class-1.png" width="100%"></td><td><img src="${CHANNEL}-failure-class-2.png" width="100%"></td></tr>
<tr><td><img src="${CHANNEL}-failure-class-3.png" width="100%"></td><td><img src="${CHANNEL}-failure-class-4.png" width="100%"></td></tr>
</table>
</p>

    <hr>
    <address><a href="mailto:gavin.mccance NOSPAM cern.ch">Gavin McCance</a></address>
  </body>
</html>

EOF
