# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Wrapper for webpages for SC3
# Will default to 06.00 UTC 
# Otherwise run as: ./sc3-summary.sh "2005-07-03 08:00:00"

if [ $# == 1 ]; then
    ds=$1
    runmanual=1
else
    # Date is today. Time is 08.00 CET / 06.00 UTC
    today=`date +%Y-%m-%d`
    time="06:00:00"
    ds="$today $time"
    runmanual=0
fi
echo Running on date: $ds

CHANNELS=""
for i in `ls -1d /var/tmp/glite-url-copy-sc3/*failed`; do CHANNELS="${CHANNELS} `basename $i | sed s'/failed//g'`"; done

#CHANNELS="CERN-CERN CERN-SARA CERN-RAL CERN-INFN CERN-IN2P3 CERN-BNL CERN-GRIDKA CERN-TRIUMF"

SHORTDATE=`date -d "${ds}" +%Y%m%d%H%M%S`

if [ $runmanual -eq 1 ] ; then
    PLOTDIR=manual/dailyplots-${SHORTDATE}
else
    PLOTDIR=dailyplots-${SHORTDATE}
fi
if [ ! -d ${PLOTDIR} ]; then
    mkdir -p ${PLOTDIR}
fi

cat > ${PLOTDIR}/index.html <<EOF
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
  <head>
    <title>Autogenerated summary plots ending ${ds} UTC</title>
  </head>

  <body>
    <h2>Autogenerated summary plots ending ${ds} UTC</h2>

A quick summary plot will go here, to identify easily which channels had problems.

<h3>Plots per channel</h3>

<p>
<ul>
EOF

for i in $CHANNELS; do
   ./makeplots.sh "$ds" $i
   cat >> ${PLOTDIR}/index.html <<EOF
<li><a href="${i}.html">Channel ${i}</a> for period ending ${ds}
EOF
done


cat >> ${PLOTDIR}/index.html <<EOF
</ul>
    <hr>
    <address><a href="mailto:gavin.mccance NOSPAM cern.ch">Gavin McCance</a></address>
  </body>
</html>
EOF

if [ $runmanual -eq 1 ]; then
    mv -f ${PLOTDIR} /afs/cern.ch/project/gd/www/sc3/fts/plots/manual/
else
    mv -f ${PLOTDIR} /afs/cern.ch/project/gd/www/sc3/fts/plots/
    rm -f /afs/cern.ch/project/gd/www/sc3/fts/plots/today
    ln -s /afs/cern.ch/project/gd/www/sc3/fts/plots/${PLOTDIR} today
fi
