# Python module for log file extraction and summary functions
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from time import *
import datetime
from sys import *
from calendar import timegm
import re
import os

class FinalLogEntry:
    
    def __init__(self, verb=False):
        self.verbose=verb
        self.lineparse = re.compile("(?P<time>\d{4}-\d{2}-\d{2}\s\d{2}:\d{2}:\d{2},\d{3})(?P<message>.*)",
                                    re.IGNORECASE |
                                    re.VERBOSE)
        self.surlparse = re.compile(r"""(?P<surl>srm://.*)""",re.IGNORECASE | re.VERBOSE)            
        self.finalmessageparse = re.compile("(?:.*FINAL:)(?P<class>\w*):(?P<message>.*)", re.IGNORECASE | re.VERBOSE)
        self.subclassparse = re.compile("(SRMPUTDONE|SRMGETDONE|SRMGET|SRMPUT|TRANSFER)")
        self.startParse = re.compile(".*?STATUS:START")
        self.doneParse = re.compile(".*?STATUS:DONE")
        self.beginParse = re.compile(".*?STATUS:BEGIN:(?P<what>.*)")
        self.endParse = re.compile(".*?STATUS:END:(?P<what>.*)")       
        self.turlparse = re.compile("TURL\s(?P<type>.*?)\s=\s(?P<turl>.*)")
        self.turlHostParser = re.compile("gsiftp://(?P<hostname>.*?)(:\d{1,4})?/{1,2}.*")

        
    def parseFromFile(self, filename, startDateQuickExit=None, endDateQuickExit=None):
        "Read a file and extract useful things from it"
        thonFile = open(filename,"r")
        self.filename = filename
        self.shortFilename = os.path.basename(filename)
        self.source_surl = None
        self.dest_surl = None
        for line in thonFile.readlines():
            parsed = self.parseTheLine(line)
            if parsed.has_key('surl'):
                if self.source_surl == None:
                    self.source_surl = parsed['surl']
                elif self.dest_surl == None:
                    self.dest_surl = parsed['surl']
                else:
                    raise ParsingError
            if parsed.has_key('line'):
                time = parsed['line'][0]
                message = parsed['line'][1]
                finalMatch = self.finalmessageparse.search(message)
                if finalMatch != None:
                    self.handleFinal(finalMatch, time)
                else:
                    beginMatch = self.beginParse.search(message)
                    if beginMatch != None:
                        self.handleBegin(beginMatch, time)
                    else:
                        endMatch = self.endParse.search(message)
                        if endMatch != None:
                            self.handleEnd(endMatch, time) 
                        else:
                            startMatch = self.startParse.search(message)
                            if startMatch != None:
                                self.startTime = self.parseTime(time)
                                if startDateQuickExit != None and endDateQuickExit != None:
                                    if self.startTime > endDateQuickExit or self.startTime < startDateQuickExit:
                                        return
                            else:
                                doneMatch = self.doneParse.search(message)
                                if doneMatch != None:
                                    self.doneTime = self.parseTime(time)
                                    
            if parsed.has_key('other'):
                turl = self.turlparse.search(parsed['other'])
                if turl != None:
                    if turl.group('type').strip() == 'source':
                        self.source_turl=turl.group('turl')
                    elif turl.group('type').strip() == 'dest':
                        self.dest_turl=turl.group('turl')    
             
        thonFile.close()



    def parseTheLine(self, line):
        line_match = self.lineparse.search(line)
        if line_match != None:
            time_match = line_match.group('time')
            message_match = line_match.group('message')
            #print time_match, message_match
            return {'line': (time_match, message_match)}
        else:
            surl_match = self.surlparse.search(line)
            if surl_match != None:
                surl = surl_match.group('surl')
                #print "Found surl: " + surl
                return {'surl': surl}
            else:
                return {'other': line}



    def parseTime(self, thetime):
        self.timeString=thetime
        maintime = thetime[:-4].strip()
        millis = 1000*int(thetime[-3:].strip())
        s = strptime(maintime, "%Y-%m-%d %H:%M:%S")
        thetime = datetime.datetime(s.tm_year, s.tm_mon, s.tm_mday, s.tm_hour, s.tm_min, s.tm_sec, millis, UTC())
        return thetime

    def parseTimeNoMillis(self, shorttime):
        millitime = shorttime + ",000"
        thetime = self.parseTime(millitime)
        return thetime

    def handleFinal(self, finalMatch, time):
        self.finalclass = finalMatch.group('class');
        self.message = finalMatch.group('message').strip()
        self.finalTime = time
        if self.finalclass == 'ABORTED' or self.finalclass == 'ABORT':
            submatch = self.subclassparse.search(self.message)
            if submatch != None:
                self.subclass = submatch.group()
            else:
                self.subclass = None
        if self.finalclass == 'SUCCESS':
            self.classcode = 1
        elif self.finalclass == 'SRM_DEST':
            self.classcode = 2
        elif self.finalclass == 'SRM_SOURCE':
            self.classcode = 3
        elif self.finalclass == 'TRANSFER':
            self.classcode = 4
        elif self.finalclass == 'ABORTED' or self.finalclass == 'ABORT':
            if self.subclass == 'SRMGET' or self.subclass == 'SRMGETDONE':
                self.classcode = 13
            elif self.subclass == 'SRMPUT' or self.subclass == 'SRMPUTDONE':
                self.classcode = 12
            else:
                self.classcode = 5
        elif self.finalclass == 'SRM_DONE_DEST':
            self.classcode = 6
        elif self.finalclass == 'SRM_DONE_SOURCE':
            self.classcode = 7
        elif self.finalclass == 'TRANSPORT':
            self.classcode = 8
        elif self.finalclass == 'NETWORK':
            self.classcode = 9
        elif self.finalclass == 'SYNTAX':
            self.classcode = 10
        elif self.finalclass == 'SECURITY':
            self.classcode = 11
        else:
            self.classcode = 0


    def handleBegin(self, beginMatch, time):                                    
        type = beginMatch.group('what').strip()
        if type == 'SRM_GET':
            self.srmGetStart = self.parseTime(time)
        elif type == 'SRM_PUT':
            self.srmPutStart = self.parseTime(time)
        elif type == 'TRANSFER':
            self.transferStart = self.parseTime(time)
        elif type == 'SRM_GETDONE':
            self.srmGetDoneStart = self.parseTime(time)
        elif type == 'SRM_PUTDONE':
            self.srmPutDoneStart = self.parseTime(time)                  

    def handleEnd(self, endMatch, time):                                    
        type = endMatch.group('what').strip()
        if type == 'SRM_GET':
            self.srmGetEnd = self.parseTime(time)
        elif type == 'SRM_PUT':
            self.srmPutEnd = self.parseTime(time)
        elif type == 'TRANSFER':
            self.transferEnd = self.parseTime(time)
        elif type == 'SRM_GETDONE':
            self.srmGetDoneEnd = self.parseTime(time)
        elif type == 'SRM_PUTDONE':
            self.srmPutDoneEnd = self.parseTime(time)                 

    def turlSouceHost(self):
        hmatch = self.turlHostParser.search(self.source_turl)
        if hmatch != None:
            return hmatch.group('hostname')

    def turlDestHost(self):
        hmatch = self.turlHostParser.search(self.dest_turl)
        if hmatch != None:
            return hmatch.group('hostname')


class FilesInRange:
    def __init__(self, startdate=None, enddate=None):
        if startdate == None: return
        # dirty dirty statics
        self.startDate = FinalLogEntry.parseTimeNoMillis(FinalLogEntry(),statedate)
        self.endDate = FinalLogEntry.parseTimeNoMillis(FinalLogEntry(),enddate)
        self.timeDelta = self.endDate-self.startDate
    def setRangeOffset(self, enddate, hourOffset):
        self.timeDelta = datetime.timedelta(hours=int(hourOffset))
        self.endDate = FinalLogEntry.parseTimeNoMillis(FinalLogEntry(),enddate)
        self.startDate = self.endDate - self.timeDelta
    def listOfFilesInRange(self, directory):
        "Return a list of all files with submit times in the given time range"
        dirlist = os.listdir(directory)
        entry = FinalLogEntry()
        fileList = []
        for x in dirlist:
            entry.parseFromFile(directory + "/" + x, self.startDate, self.endDate)
            if entry.startTime <= self.endDate and entry.startTime >= self.startDate:
                fileList.append(x)
        return fileList
    def entriesInRange(self, directory, withIndicator=False):
        "Return a list of entries with submit times in the given time range"
        dirlist = os.listdir(directory)
        entryList = []
        if withIndicator:
            print "Scanning:  " + directory
            for x in dirlist:
                entry = FinalLogEntry()
                entry.parseFromFile(directory + x, self.startDate, self.endDate)
                if entry.startTime <= self.endDate and entry.startTime >= self.startDate:
                    entryList.append(entry)
                    stdout.write("o")
                else:
                    stdout.write("x")
                stdout.flush()
            print ""
        else:
            for x in dirlist:
                entry = FinalLogEntry()
                entry.parseFromFile(directory + x, self.startDate, self.endDate)
                if entry.startTime <= self.endDate and entry.startTime >= self.startDate:
                    entryList.append(entry)
        return entryList


class UTC(datetime.tzinfo):
    """UTC"""

    def utcoffset(self, dt):
        return datetime.timedelta(0)

    def tzname(self, dt):
        return "UTC"

    def dst(self, dt):
        return datetime.timedelta(0)


# and now the tests
if __name__ == "__main__":
    
    # test file read bad
    entry = FinalLogEntry()
    entry.parseFromFile("../../test/config/summary/bad-url-copy.log")
    assert entry.source_surl=="srm://castorgridsc.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/grid/dteam/storage/transfer-test/smalltest/small-file-006.dat"
    assert entry.dest_surl=="srm://srm1.tier1.sara.nl:8443/srm/managerv1?SFN=/pnfs/tier1.sara.nl/data/dteam/testdestfile-glite-1.1.1-126-2a3416a3-e0f3-4d04-a3d0-90f0f0db3dc6.dat"
    assert entry.classcode==3
    assert entry.finalclass=="SRM_SOURCE"

    #test file read good
    entry = FinalLogEntry()
    entry.parseFromFile("../../test/config/summary/good-url-copy.log")
    assert entry.source_surl=="srm://castorgridsc.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/grid/dteam/storage/transfer-test/1gig/file-208.dat"
    assert entry.dest_surl=="srm://radiantservice.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/grid/dteam/storage/transfer-test/castor2/tmp/file-208-615c527c-3ed9-42ae-9dfd-dd8a02b663a7.dat"
    assert entry.classcode==1
    assert entry.finalclass=="SUCCESS"
    assert entry.source_turl=="gsiftp://lxshare032d.cern.ch//shift/lxshare032d/data01/cg/stage/file-208.dat.36358"
    assert entry.dest_turl=="gsiftp://lxshare219d.cern.ch//castor/cern.ch/grid/dteam/storage/transfer-test/castor2/tmp/file-208-615c527c-3ed9-42ae-9dfd-dd8a02b663a7.dat"
    assert entry.doneTime.microsecond==822000
    assert entry.srmPutDoneStart.microsecond==363000
    assert entry.srmGetStart.microsecond==472000
    assert entry.turlSouceHost()=="lxshare032d.cern.ch"
    assert entry.turlDestHost()=="lxshare219d.cern.ch"

    # test FilesInRange
    
