#!/usr/bin/python
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import cx_Oracle

def connectDB():
   dbfile = open('tellme.conf')
   try:
      slurp=dbfile.readlines()
   finally:
      dbfile.close()
   dbuser=slurp[0].rstrip()
   dbpasswd=slurp[1].rstrip()
   dbconnstring=slurp[2].rstrip()
   dbconn = cx_Oracle.connect(dbuser,dbpasswd,dbconnstring)
   return dbconn 

def processdbex(detail, stat):
   print 'Error: %s' % detail
   print 'From query: %s' % stat
   return

class JobFinder:
   def __init__(self, cursor):
      self.cursor = cursor

   def findSurl(self, filepart, srcORdest, exact=1):
      if srcORdest == 'dest':
         scope = 'dest_surl'
      elif srcORdest == 'src':
         scope = 'source_surl'
      else:
         print "Error: you must seach on either 'src' or 'dest'!"
         return


      try:
         if exact:
            self.cursor.prepare("""SELECT T.job_id from t_job T, t_file F where F.%s = :surl
                                   AND T.job_id = F.job_id""" % scope)
            self.cursor.execute(None, (filepart,))
         else:
            self.cursor.prepare("""SELECT T.job_id from t_job T, t_file F where F.%s like :surl
                                   AND T.job_id = F.job_id""" % scope)
            self.cursor.execute(None, ('%' + filepart + '%',))
                                    
      except cx_Oracle.DatabaseError, detail:
         processdbex(detail, self.cursor.statement);
         return  
      rows = self.cursor.fetchall()	
      if self.cursor.rowcount == 0:
         print "No jobs found for %s containing '%s'" % (scope, filepart)
      else:
         self.matchedJobs = []
         for row in rows:
            self.matchedJobs.append(row[0]);
         return self.matchedJobs;

class JobSummary:
   def __init__(self, cursor, jobID):
      self.cursor = cursor
      self.jobID = jobID
	
   def fetchSummary(self):
      try:
         self.cursor.prepare("SELECT job_state, channel_name, submit_time, vo_name, priority, user_dn from t_job WHERE job_id=:jid")
         self.cursor.execute(None, (self.jobID,))
      except cx_Oracle.DatabaseError, detail:
         processdbex(detail, self.cursor.statement);
         return  
      row = self.cursor.fetchone()	
      if self.cursor.rowcount == 0:
         print "Job '%s' does not exist!" % self.jobID
      else:
         self.job_state, self.channel_name, self.submit_time, self.vo_name, self.priority, self.user_dn  = row[0], row[1], row[2], row[3], row[4], row[5]
      try:
         self.cursor.prepare("SELECT COUNT(file_id) from t_file WHERE job_id=:jid")
         self.cursor.execute(None, (self.jobID,))
      except cx_Oracle.DatabaseError, detail:
         processdbex(detail, self.cursor.statement);  
         return
      row = self.cursor.fetchone()	
      if self.cursor.rowcount == 0:
         print "Job '%s' does not exist!" % self.jobID
      else:
         self.numFiles = row[0]

   def __repr__(self):
      print self.__dict__
      return self.jobID;

   def printSummary(self):
      print 'Job ID: %s' % self.jobID;
      print '  DN: %s' % self.user_dn
      print '  VO: %s' % self.vo_name
      print '  Numfiles: %s' % self.numFiles
      print '  Channel: %s' % self.channel_name
      print '  Priority: %i' % self.priority
      print '  Submit time: %s' % self.submit_time
      print '  State: %s' % self.job_state

class FileSummary:
   def __init__(self, cursor, jobID):
      self.cursor = cursor
      self.jobID = jobID;
	
   def fetchSummary(self):

      # get all the filkes for the job
      try:
         self.cursor.prepare("""SELECT file_id, file_state, source_surl, dest_surl, finish_time  
                                FROM t_file WHERE job_id = :jid""")
         self.cursor.execute(None, (self.jobID,))
      except cx_Oracle.DatabaseError, detail:
         processdbex(detail, self.cursor.statement);
         return  
      rows = self.cursor.fetchall()	
      if self.cursor.rowcount == 0:
         print "Job '%s' does not exist!" % self.jobID
         return
      else:
         # store file tuple indexed by source surl, dest surl, fileid
         # create dict to store the transfers
         self.sourceSurlDict = {}
         self.destSurlDict = {}
         self.fileIdDict = {}
         self.transfersDict = {}
         for row in rows:
            tup = row[:]
            self.sourceSurlDict[row[2]] = tup
            self.destSurlDict[row[3]] = tup
            self.fileIdDict[row[0]] = tup
            self.transfersDict[row[0]] = None
         
      for fileid in self.transfersDict.keys():
         try:
            self.cursor.prepare("""SELECT request_id, transfer_state, source_turl, dest_turl,
                                   transfer_time, duration, bytes_written, reason_class, reason
                                   FROM t_transfer WHERE file_id = :fileid ORDER BY transfer_time""")
            self.cursor.execute(None, (fileid,))
         except cx_Oracle.DatabaseError, detail:
            processdbex(detail, self.cursor.statement);
            return  
         rows = self.cursor.fetchall()	
         if self.cursor.rowcount == 0:
            print "FileID has no transfer attempts yet!"
            return
         else:
            trans = []
            for row in rows:
               tup = row[:]
               trans.append(tup)
            self.transfersDict[fileid] = trans
 
   def printSelectedFiles(self, fileIDList, withFileInfo=1, withTransfers=1):
      for fileid in fileIDList:
         self.printFileSummary(fileid, withFileInfo, withTransfers);
         print
  
   def printAllFiles(self, withFileInfo=1, withTransfers=1):
      for fileid in self.fileIdDict.keys():
         self.printFileSummary(fileid, withFileInfo, withTransfers);

   def printDestFilesMatching(self, destFilePattern, withFileInfo=1, withTransfers=1):
      matchlist = []
      for match in self.destSurlDict.keys():
         if match.find(destFilePattern) != -1:
            matchlist.append(self.destSurlDict[match][0])
      self.printSelectedFiles(matchlist, withFileInfo, withTransfers)
      
   def printSourceFilesMatching(self, srcFilePattern, withFileInfo=1, withTransfers=1):
      matchlist = []
      for match in self.sourceSurlDict.keys():
         if match.find(srcFilePattern) != -1:
            matchlist.append(self.sourceSurlDict[match][0])
      self.printSelectedFiles(matchlist, withFileInfo, withTransfers)
      
   def printFileSummary(self, fileID, withFileInfo=1, withTransfers=1):
      print
      print 'FileID: %i' % fileID
      if withFileInfo:
         print '  Source: %s' % self.fileIdDict[fileID][2]
         print '  Dest  : %s' % self.fileIdDict[fileID][3]
      print '  State : %s' % self.fileIdDict[fileID][1]
      print '  Finish time: %r' % self.fileIdDict[fileID][4]
      if withTransfers:
         cnt=1
         for transfer in self.transfersDict[fileID]:
            print
            print '  Transfer attempt: %i' % cnt
            cnt=cnt+1
            print '    Time: %r' % transfer[4]
            print '    State: %r' % transfer[1]
            print '    ReqID: %r' % transfer[0]
            print '    Reason class: %r' % transfer[7]
            print '    Reason class: %r' % transfer[8]
            if withFileInfo:
               print '    Source TURL: %r' % transfer[2]
               print '    Dest TURL  : %r' % transfer[3]
            print '    Duration(s): %r' % transfer[5]
            print '    Bytes written: %r' % transfer[6]
                                   
if __name__ == '__main__':
   
   dbconn = connectDB()
   curs = dbconn.cursor()

   js = JobSummary(curs, "eb2c602c-1d4e-11db-9ec6-c9926a2c9d0b")
   js.fetchSummary()
   fs = FileSummary(curs, "eb2c602c-1d4e-11db-9ec6-c9926a2c9d0b")
   fs.fetchSummary()
   
   js.printSummary()
   print
   #fs.printAllFiles(withFileInfo=1, withTransfers=1)
   #fs.printSelectedFiles([9645782, 9645783])
   fs.printSourceFilesMatching('T0.DQ2.AOD._7d4782af-1d3b-455a-b157-d72f28fe3db2.deleteable')

   #jf = JobFinder(curs)
   #lis = jf.findSurl('T0.DQ2.AOD._7d4782af-1d3b-455a-b157-d72f28fe3db2.deleteable', 'dest')
   #print lis
