#!/usr/bin/python
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import jobfilesummarylib
import getopt
import sys

class JobSearchCli:

   def usage(self, msg):
      if msg:
         print msg
         print
      print "Usage: " + sys.argv[0] + " -x -i -h -n -f 'filestring'"
      print
      print  "Options:"
      print "\tJobID\t\t\tThe jobID to search for"
      print "\t-h | --help\t\tThis help page"
      print "\t-x | --expand\t\tShow also the file details for each matching job rather than just the job details"
      print "\t-i | --hidefiles\tHide the filenames for more compact output; only relevant if -x is set"
      print "\t-n | --notransfers\tHide the transfer details; only relevant if -x is set"
      print "\t-f | --file\t\tDisplay file details only for file containing this string"
      print "\t-z | --fuzzy\t\tAllow the string specified in -f to be a substring match (expensive on FTS database)"
      print "\t-s | --src\t\tMatch the file specified in -f on the source SURL"
      print "\t-d | --dest\t\tMatch the file specified in -f on the destination SURL"
  
   def parseCLI(self):
      try:
         opts, args = getopt.getopt(sys.argv[1:], 'xihnsdf:', ['expand', 'hidefiles', 'help', 'notransfers', 'src', 'dest', 'file='])
      except getopt.error, msg:
         self.usage(msg)
         sys.exit(1)

      self.scope=None
      self.restrictFile = None
      self.withFiles = 1
      self.withTransfers=1
      self.expand=0
      self.exact=1
      for opt, arg in opts:
         if opt in ('-h', '--help'):
            self.usage(0)
            sys.exit(0)
         elif opt in ('-f', '--file'):
            self.restrictFile = arg
         elif opt in ('-s', '--src'):
            self.scope='src'
         elif opt in ('-d', '--dest'):
            self.scope='dest'
         elif opt in ('-n', '--notransfers'):
            self.withTransfers=0
         elif opt in ('-i', '--hidefiles'):
            self.withFiles=0
         elif opt in ('-x', '--expand'):
            self.expand = 1
         elif opt in ('z', '--fuzzy'):
            self.exact = 0
         
      if self.restrictFile  == None:
         self.usage('You must specify a file substring to search for using the -f parameter')
         sys.exit(1)
         
      if self.scope == None:
         self.usage("You must specify whether you are matching the source or destination SURL (use the --src | --dest option)")
         sys.exit(1)

   def do(self):
      dbconn = jobfilesummarylib.connectDB()
      curs = dbconn.cursor()
      jf = jobfilesummarylib.JobFinder(curs)
      jobs = jf.findSurl(self.restrictFile, self.scope, self.exact)
      if not jobs:
         print "No jobs were found with " + self.scope + " files containing the given substring."
         sys.exit(1)

      print "Found jobs with " + self.scope + " files containing the given substring:"
      for jobid in jobs:
         print jobid
      print

      for jobid in jobs:
         js = jobfilesummarylib.JobSummary(curs, jobid)
         js.fetchSummary()
         js.printSummary()
         print
         if self.expand:
            fs = jobfilesummarylib.FileSummary(curs, jobid)
            fs.fetchSummary()
            if self.scope=='src':
               fs.printSourceFilesMatching(self.restrictFile, withFileInfo=self.withFiles, withTransfers=self.withTransfers);
            elif self.scope=='dest':
               fs.printDestFilesMatching(self.restrictFile, withFileInfo=self.withFiles, withTransfers=self.withTransfers);

if __name__ == '__main__':
  js = JobSearchCli()
  js.parseCLI()
  js.do()
