#!/usr/bin/python
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import jobfilesummarylib
import getopt
import sys

class JobSummaryCli:

   def usage(self, msg):
      if msg:
         print msg
         print
      print "Usage: " + sys.argv[0] + " -h -n [-f 'filestring' --src|--dest] jobID"
      print
      print  "Options:"
      print "\tJobID\t\t\tThe jobID to search for"
      print "\t-h | --help\t\tThis help page"
      print "\t-i | --hidefiles\tHide the filenames for more compact output"
      print "\t-n | --notransfers\tHide the transfer details"
      print "\t-f | --file\t\tDisplay file details only for file containing this substring"
      print "\t-s | --src\t\tMatch the file specificd in -f on the source SURL"
      print "\t-d | --dest\t\tMatch the file specificd in -f on the destination SURL"
  
   def parseCLI(self):
      try:
         opts, args = getopt.getopt(sys.argv[1:], 'ihnsdf:', ['hidefiles', 'help', 'notransfers', 'src', 'dest', 'file='])
      except getopt.error, msg:
         self.usage(msg)
         sys.exit(1)

      self.scope=None
      self.restrictFile = None
      self.withFiles = 1
      self.withTransfers=1
      for opt, arg in opts:
         if opt in ('-h', '--help'):
            self.usage(0)
            sys.exit(0)
         elif opt in ('-f', '--file'):
            self.restrictFile = arg
         elif opt in ('-s', '--src'):
            self.scope='src'
         elif opt in ('-d', '--dest'):
            self.scope='dest'
         elif opt in ('-n', '--notransfers'):
            self.withTransfers=0
         elif opt in ('-i', '--hidefiles'):
            self.withFiles=0
         
      self.jobID = args[0]

      if self.restrictFile != None and self.scope == None:
           self.usage("To restrict with the --file option, you must specify whether you are matching the "
                       + "source or destination SURL (use the --src | --dest option)")

   def do(self):
      dbconn = jobfilesummarylib.connectDB()
      curs = dbconn.cursor()
      js = jobfilesummarylib.JobSummary(curs, self.jobID)
      fs = jobfilesummarylib.FileSummary(curs, self.jobID)
      js.fetchSummary()
      fs.fetchSummary()
      if self.restrictFile == None:
         print "Printing all information for jobID <" + self.jobID + ">\n"
         js.printSummary()
         print
         print
         fs.printAllFiles(withFileInfo=self.withFiles, withTransfers=self.withTransfers)
      else:
         if self.scope=='src':
            print "Printing information for jobID <" + self.jobID + ">" + " with source file containing '" + self.restrictFile + "'\n"
            js.printSummary()
            print
            print
            fs.printSourceFilesMatching(self.restrictFile, withFileInfo=self.withFiles, withTransfers=self.withTransfers);
         elif self.scope=='dest':
            print "Printing information for jobID <" + self.jobID + ">" + " with dest file containing '" + self.restrictFile + "'\n"
            js.printSummary()
            print
            print
            fs.printDestFilesMatching(self.restrictFile, withFileInfo=self.withFiles, withTransfers=self.withTransfers);

if __name__ == '__main__':
  js = JobSummaryCli()
  js.parseCLI()
  js.do()
